# Charts add-on

## What is the Charts add-on?

The Charts add-on provides integration of the [Apache ECharts](https://echarts.apache.org/) JavaScript library in Jmix UI.
Apache ECharts is a free, powerful OpenSource charting and visualization library.

## Installation

For automatic installation Jmix Marketplace can be used.

For manual installation, do the following:

1. Add dependency to your `build.gradle` file:
    ```groovy
   implementation 'io.jmix.charts:jmix-charts-flowui-starter'
   ```
2. To enable using the component in a view, declare the `charts` namespace in the root element of the view XML descriptor:
    ```xml
   <view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
        <!-- view code -->
   </view>
   ```
   Or just use the component palette in the Jmix UI visual designer:
   <img alt="img.png" src="img.png"/>
   In this case, the declaration of the space will occur automatically.

## What is what?
### Chart component

Basically, chart is a component for showing data in graphical form.
In the context of the **Jmix Framework** `chart` is a visual component, that implements the behavior of the Vaadin component.

In other words: you can set the visibility and size properties, like for `textArea`, for example.

In the context of the **Apache ECharts JavaScript** library, a `chart` is a 'component-constructor', that can be configured using options.
You can view the full range of available options in the [Apache ECharts options documentation](https://echarts.apache.org/en/option.html).

During implementation, we tried to maintain transparency and continuity of the origin documentation so that you can use a continuous workflow.

What does it mean? You can use the **original documentation** to find the properties you need, and with a 95% chance you will **find this property in the component's Java API or XML markup**.

A huge number of chart component properties are available for customization. You can customize the title, legend, series to be displayed and much more.

The component is completely integrated into the Framework and can be created and customized both declaratively, through XML markup, and programmatically, inside the view controller.

> **Note**: the easiest way to write your own chart is to base it on an existing one. See [Apache ECharts Examples](https://echarts.apache.org/examples/en/index.html).
> You can find an example that suits you and modify it. After this, add the properties to the component.

### DataSet

`DataSet` component is the recommended way to provide data to a chart. All series added to the chart will use the data set.
The only series that requires its own data set is the `GaugeSeries`.

You can configure DataSet using XML markup:
```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <data>
        <collection id="dateValueDc"
                    class="com.company.chartsaddon.entity.DateValue">
            <fetchPlan extends="_local"/>
            <loader id="dateValueLoader">
                <query>
                    <![CDATA[select e from DateValue e order by e.date]]>
                </query>
            </loader>
        </collection>
    </data>
    <facets>
        <dataLoadCoordinator auto="true"/>
    </facets>
    <layout>
        <charts:chart id="chart" width="100%" height="100%">
            <charts:xAxes>
                <charts:xAxis/>
            </charts:xAxes>
            <charts:yAxes>
                <charts:yAxis/>
            </charts:yAxes>

            <charts:dataSet>
                <charts:source dataContainer="dateValueDc"
                               categoryField="date" valueFields="value"/>
            </charts:dataSet>

            <charts:series>
                <charts:line id="line"/>
            </charts:series>
        </charts:chart>
    </layout>
</view>
```
In this example, the `DataSet` is bound to a standard collection container.

Programmatically configuration is also available:
```java
    @ViewComponent
    protected Chart chart;
    @ViewComponent
    protected CollectionContainer<DateValue> dateValueDc;

    @Subscribe
    protected void onInit(InitEvent event) {
       chart.setDataSet(
               new DataSet().withSource(
                       new DataSet.Source<EntityDataItem>()
                               .withDataProvider(new ContainerChartItems<>(dateValueDc))
                               .withCategoryField("date")
                               .withValueField("value")
               )
       );
    }
```

### Main properties

#### Title
The title component is used to display the chart title.

Example:
```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart id="chart" height="75%" width="75%">
            <charts:title text="Title text"
                          subtext="Subtitle text" top="0" left="35%" textAlign="CENTER">
                <charts:textStyle fontWeight="bold"/>
            </charts:title>
           
           <!-- other chart options-->
        </charts:chart>
    </layout>
</view>
```

Result: ![img_1.png](img_1.png)

#### Legend
Legend component shows symbol, color and name of different series. You can click legends to toggle displaying
series in the chart. The legend will be automatically generated based on the data provided.

Use `<charts:legend/>` tag to configure.

Example:
```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart id="chart" height="75%" width="75%">
            <charts:legend/>
           
           <!-- other chart options-->
        </charts:chart>
    </layout>
</view>
```

Result: ![img_2.png](img_2.png)

#### DataZoom

There are two types of data zoom feature.

* Inside data zoom functionalities is embedded inside coordinate systems, enable user to zoom or roam coordinate system by mouse dragging, mouse move or finger touch (in touch screen).
* Slider type data zoom component provides functions like data thumbnail, zoom, brush to select, drag to move, click to locate. More detailed information is provided in the documentation.

Example:
```xml
<view xmlns="http://jmix.io/schema/flowui/view" 
      xmlns:charts="http://jmix.io/schema/charts/ui">
   <layout>
      <charts:chart id="chart" width="100%" height="100%">
         <!-- other chart options -->
         
         <charts:dataZoom>
            <charts:sliderDataZoom orientation="HORIZONTAL"/>
            <charts:sliderDataZoom orientation="VERTICAL" right="5%"/>
            <charts:insideDataZoom/>
         </charts:dataZoom>

         <charts:series>
            <charts:line id="line">
               <!-- line options -->
            </charts:line>
         </charts:series>
      </charts:chart>
   </layout>
</view>
```
Result: ![img_3.png](img_3.png)

#### Tooltip

Tooltip can be configured on different places.
The `formatter` attribute is the most important attribute of this component. The `formatter` can be
specified using a JavaScript function or template. See: [tooltip documentation](https://echarts.apache.org/en/option.html#tooltip.formatter).

Example:
```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart id="chart" width="100%" height="100%">
            <charts:tooltip trigger="AXIS">
                <charts:formatterFunction>
                    <![CDATA[
                        function (params) {
                              var tar = params[1];

                              return '<b>Year:</b> ' + tar.name + '<br/><hr/>' +
                              '<b>Cars:</b> ' + tar.value.cars + '<br/>' +
                              '<b>Motorcycles:</b> ' + tar.value.motorcycles + '<br/>' +
                              '<b>Bicycles:</b> ' + tar.value.bicycles + '<br/>';
                            }
                    ]]>
                </charts:formatterFunction>
                <charts:axisPointer type="SHADOW"/>
            </charts:tooltip>

            <!-- other chart options -->

            <charts:series>
                <charts:bar name="Car sells"/>
                <charts:bar name="Motorcycles sells"/>
                <charts:bar name="Bicycles sells"/>
            </charts:series>
        </charts:chart>
    </layout>
</view>
```

Result: ![img_4.png](img_4.png)

#### Export

The easiest way to export a chart is to use `saveAsImage` feature in the `toolbox` component.

Example:
```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart id="chart" width="100%" height="100%">
            <!-- other chart options -->

            <charts:toolbox>
                <charts:features>
                    <charts:saveAsImage type="PNG"/>
                </charts:features>
            </charts:toolbox>

            <charts:series>
                <!-- series -->
            </charts:series>
        </charts:chart>
    </layout>
</view>
```

Result:![img_5.png](img_5.png)

#### NativeJson

An additional JSON options can be set as a string. This JSON can override options loaded from XML or from component
Java API if they overlap. It is recommended to use `nativeJson` if the component API or XML is not enough for chart configuration.

> **Note:** `nativeJson` will be merged with existing properties.

Example:
```xml

<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart height="100%" width="100%">
            <charts:title text="Title"/>
            <charts:nativeJson>
                <![CDATA[{
                    "title": {
                        "subtext": "subtext"
                    }
                 }
        ]]>
            </charts:nativeJson>
        </charts:chart>
    </layout>
</view>
```
Result:

![img_6.png](img_6.png)

### Connecting data

There are two `DataProvider` implementations for a chart component: `ContainerChartItems` and `ListChartItems`.
All changes in associated items are listened to and passed to the client-side to update the data.

#### `ListChartItems`
Code snippet:
```java
    protected DataSet createDataSet() {
        dataItemListChartItems = new ListChartItems<>();
        dataItemListChartItems.addItems(
                new MapDataItem(Map.of("year", 2015, "cars", 10, "motorcycles", 50)),
                new MapDataItem(Map.of("year", 2016, "cars", 15, "motorcycles", 102)),
                new MapDataItem(Map.of("year", 2017, "cars", 1, "motorcycles", 70))
        );

        return new DataSet()
                .withSource(
                        new DataSet.Source<MapDataItem>()
                                .withCategoryField("year")
                                .withValueFields("cars", "motorcycles")
                                .withDataProvider(dataItemListChartItems)
                );
    }
```

There are three date items in total:
* EntityDataItem - takes an instance of any entity from which the data for the chart will be taken.
* MapDataItem - is a set of key-value pairs from which the data for the chart will be taken.
* SimpleDataItem - takes an instance of any public class from which the data for the chart will be taken.

#### `ContainerChartItems`

XML example:
```xml

<view xmlns="http://jmix.io/schema/flowui/view" xmlns:charts="http://jmix.io/schema/charts/ui">
    <data>
        <collection id="dateValueDc"
                    class="com.company.chartsaddon.entity.DateValue">
            <fetchPlan extends="_local"/>
            <loader id="dateValueLoader">
                <query>
                    <![CDATA[select e from DateValue e order by e.date]]>
                </query>
            </loader>
        </collection>
    </data>
    <facets>
        <dataLoadCoordinator auto="true"/>
    </facets>
    <layout>
        <charts:chart id="chart" width="100%" height="100%">

            <charts:dataSet>
                <charts:source dataContainer="dateValueDc"
                               categoryField="date" valueFields="value"/>
            </charts:dataSet>

            <!-- other chart options -->
        </charts:chart>
    </layout>
</view>
```

Java example:
```java
@ViewComponent
protected Chart chart;

@Subscribe
protected void onInit(InitEvent event) {
   chart.setDataSet(
           new DataSet().withSource(
                   new DataSet.Source<EntityDataItem>()
                           .withDataProvider(new ContainerChartItems<>(dateValueDc))
           )
   );
}
```

#### GaugeSeries data binding
The only series that requires its own data set is the `GaugeSeries`.
Below is an example of specifying data for a series.

```xml
<view xmlns="http://jmix.io/schema/flowui/view"
      xmlns:charts="http://jmix.io/schema/charts/ui">
    <layout>
        <charts:chart>
            <charts:tooltip formatter="{a} &lt;br/&gt;{b} : {c}%"/>

            <charts:series>
                <charts:gauge name="Pressure">
                    <charts:data>
                        <charts:dataItem value="50" name="SCORE"/>
                    </charts:data>
                    <charts:detail formatter="{value}"/>
                </charts:gauge>
            </charts:series>
        </charts:chart>
    </layout>
</view>
```
